# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class UpdatePropertyDefinitionRequest(BaseModel):
    """
    UpdatePropertyDefinitionRequest
    """
    display_name:  StrictStr = Field(...,alias="displayName", description="The display name of the property.") 
    property_description:  Optional[StrictStr] = Field(None,alias="propertyDescription", description="Describes the property") 
    custom_entity_types: Optional[List[StrictStr]] = Field(default=None, description="The custom entity types that properties relating to this property definition can be applied to.", alias="customEntityTypes")
    value_format:  Optional[StrictStr] = Field(None,alias="valueFormat", description="The format in which values for this property definition should be represented.") 
    __properties = ["displayName", "propertyDescription", "customEntityTypes", "valueFormat"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdatePropertyDefinitionRequest:
        """Create an instance of UpdatePropertyDefinitionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if property_description (nullable) is None
        # and __fields_set__ contains the field
        if self.property_description is None and "property_description" in self.__fields_set__:
            _dict['propertyDescription'] = None

        # set to None if custom_entity_types (nullable) is None
        # and __fields_set__ contains the field
        if self.custom_entity_types is None and "custom_entity_types" in self.__fields_set__:
            _dict['customEntityTypes'] = None

        # set to None if value_format (nullable) is None
        # and __fields_set__ contains the field
        if self.value_format is None and "value_format" in self.__fields_set__:
            _dict['valueFormat'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdatePropertyDefinitionRequest:
        """Create an instance of UpdatePropertyDefinitionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdatePropertyDefinitionRequest.parse_obj(obj)

        _obj = UpdatePropertyDefinitionRequest.parse_obj({
            "display_name": obj.get("displayName"),
            "property_description": obj.get("propertyDescription"),
            "custom_entity_types": obj.get("customEntityTypes"),
            "value_format": obj.get("valueFormat")
        })
        return _obj

UpdatePropertyDefinitionRequest.update_forward_refs()
