# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class SettlementSchedule(BaseModel):
    """
    SettlementSchedule
    """
    trade_id:  Optional[StrictStr] = Field(None,alias="tradeId") 
    settlement_date: Optional[datetime] = Field(default=None, alias="settlementDate")
    units: Optional[Union[StrictFloat, StrictInt]] = None
    bond_interest: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, alias="bondInterest")
    movement_name:  Optional[StrictStr] = Field(None,alias="movementName") 
    movement_type:  Optional[StrictStr] = Field(None,alias="movementType") 
    unsettled_units: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, alias="unsettledUnits")
    overdue_units: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, alias="overdueUnits")
    __properties = ["tradeId", "settlementDate", "units", "bondInterest", "movementName", "movementType", "unsettledUnits", "overdueUnits"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SettlementSchedule:
        """Create an instance of SettlementSchedule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if trade_id (nullable) is None
        # and __fields_set__ contains the field
        if self.trade_id is None and "trade_id" in self.__fields_set__:
            _dict['tradeId'] = None

        # set to None if movement_name (nullable) is None
        # and __fields_set__ contains the field
        if self.movement_name is None and "movement_name" in self.__fields_set__:
            _dict['movementName'] = None

        # set to None if movement_type (nullable) is None
        # and __fields_set__ contains the field
        if self.movement_type is None and "movement_type" in self.__fields_set__:
            _dict['movementType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SettlementSchedule:
        """Create an instance of SettlementSchedule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SettlementSchedule.parse_obj(obj)

        _obj = SettlementSchedule.parse_obj({
            "trade_id": obj.get("tradeId"),
            "settlement_date": obj.get("settlementDate"),
            "units": obj.get("units"),
            "bond_interest": obj.get("bondInterest"),
            "movement_name": obj.get("movementName"),
            "movement_type": obj.get("movementType"),
            "unsettled_units": obj.get("unsettledUnits"),
            "overdue_units": obj.get("overdueUnits")
        })
        return _obj

SettlementSchedule.update_forward_refs()
