# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class MarketContextSuppliers(BaseModel):
    """
    It is possible to control which supplier is used for a given asset class.  This field is deprecated in favour of market data rules, which subsumes its functionality.  # noqa: E501
    """
    commodity:  Optional[StrictStr] = Field(None,alias="Commodity") 
    credit:  Optional[StrictStr] = Field(None,alias="Credit") 
    equity:  Optional[StrictStr] = Field(None,alias="Equity") 
    fx:  Optional[StrictStr] = Field(None,alias="Fx") 
    rates:  Optional[StrictStr] = Field(None,alias="Rates") 
    __properties = ["Commodity", "Credit", "Equity", "Fx", "Rates"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MarketContextSuppliers:
        """Create an instance of MarketContextSuppliers from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MarketContextSuppliers:
        """Create an instance of MarketContextSuppliers from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MarketContextSuppliers.parse_obj(obj)

        _obj = MarketContextSuppliers.parse_obj({
            "commodity": obj.get("Commodity"),
            "credit": obj.get("Credit"),
            "equity": obj.get("Equity"),
            "fx": obj.get("Fx"),
            "rates": obj.get("Rates")
        })
        return _obj

MarketContextSuppliers.update_forward_refs()
