# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.link import Link
from lusid.models.posting_module_rule import PostingModuleRule
from lusid.models.resource_id import ResourceId
from lusid.models.version import Version

class PostingModuleResponse(BaseModel):
    """
    A Posting Module definition  # noqa: E501
    """
    href:  Optional[StrictStr] = Field(None,alias="href", description="The specific Uniform Resource Identifier (URI) for this resource at the requested effective and asAt datetime.") 
    posting_module_code:  StrictStr = Field(...,alias="postingModuleCode", description="The code of the Posting Module.") 
    chart_of_accounts_id: ResourceId = Field(alias="chartOfAccountsId")
    display_name:  StrictStr = Field(...,alias="displayName", description="The name of the Posting Module.") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="A description for the Posting Module.") 
    rules: Optional[List[PostingModuleRule]] = Field(default=None, description="The Posting Rules that apply for the Posting Module. Rules are evaluated in the order they occur in this collection.")
    status:  StrictStr = Field(...,alias="status", description="The Posting Module status. Can be Active, Inactive or Deleted. Defaults to Active.") 
    version: Optional[Version] = None
    links: Optional[List[Link]] = None
    __properties = ["href", "postingModuleCode", "chartOfAccountsId", "displayName", "description", "rules", "status", "version", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PostingModuleResponse:
        """Create an instance of PostingModuleResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of chart_of_accounts_id
        if self.chart_of_accounts_id:
            _dict['chartOfAccountsId'] = self.chart_of_accounts_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in rules (list)
        _items = []
        if self.rules:
            for _item in self.rules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['rules'] = _items
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if rules (nullable) is None
        # and __fields_set__ contains the field
        if self.rules is None and "rules" in self.__fields_set__:
            _dict['rules'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PostingModuleResponse:
        """Create an instance of PostingModuleResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PostingModuleResponse.parse_obj(obj)

        _obj = PostingModuleResponse.parse_obj({
            "href": obj.get("href"),
            "posting_module_code": obj.get("postingModuleCode"),
            "chart_of_accounts_id": ResourceId.from_dict(obj.get("chartOfAccountsId")) if obj.get("chartOfAccountsId") is not None else None,
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "rules": [PostingModuleRule.from_dict(_item) for _item in obj.get("rules")] if obj.get("rules") is not None else None,
            "status": obj.get("status"),
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None,
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj

PostingModuleResponse.update_forward_refs()
